@@ -7,13 +7,16 @@ from . import yfc_time as yfct
 from . import yfc_prices_manager as yfcp
 
 import numpy as np
+import pandas as pd
 import datetime
 from zoneinfo import ZoneInfo
 import os
+import re
 # from time import perf_counter
 
 # TODO: Ticker: add method to delete ticker from cache
 
+
 class Ticker:
     def __init__(self, ticker, session=None):
         self.ticker = ticker.upper()
@@ -60,9 +63,6 @@ class Ticker:
 
         self._debug = False
         # self._debug = True
-        self._trace = False
-        # self._trace = True
-        self._trace_depth = 0
 
     def history(self,
                 interval="1d",
@@ -82,13 +82,12 @@ class Ticker:
         debug_yfc = self._debug
         # debug_yfc = True
 
-        log_msg = "YFC: history(tkr={}, interval={}, period={}, start={}, end={}, max_age={}, adjust_splits={}, adjust_divs={})".format(self.ticker, interval, period, start, end, max_age, adjust_splits, adjust_divs)
-        if debug_yfc:
-            print("")
-            print(log_msg)
-        elif self._trace:
-            self._trace_depth += 1
-            print(" "*self._trace_depth + log_msg)
+        if start is not None or end is not None:
+            log_msg = f"Ticker::history(tkr={self.ticker} interval={interval} start={start} end={end} max_age={max_age} adjust_splits={adjust_splits}, adjust_divs={adjust_divs})"
+        else:
+            log_msg = f"Ticker::history(tkr={self.ticker} interval={interval} period={period} max_age={max_age} adjust_splits={adjust_splits}, adjust_divs={adjust_divs})"
+        if yfcu.tc is not None:
+            yfcu.tc.Enter(log_msg)
 
         td_1d = datetime.timedelta(days=1)
         exchange = self.fast_info['exchange']
@@ -98,8 +97,13 @@ class Ticker:
         dt_now = datetime.datetime.utcnow().replace(tzinfo=ZoneInfo("UTC"))
 
         # Type checks
-        if (max_age is not None) and (not isinstance(max_age, datetime.timedelta)):
-            raise Exception("Argument 'max_age' must be timedelta")
+        if max_age is not None:
+            if isinstance(max_age, str):
+                if max_age.endswith("wk"):
+                    max_age = re.sub("wk$", "w", max_age)
+                max_age = pd.Timedelta(max_age)
+            if not isinstance(max_age, (datetime.timedelta, pd.Timedelta)):
+                raise Exception("Argument 'max_age' must be Timedelta or valid string")
         if period is not None:
             if start is not None or end is not None:
                 raise Exception("Don't set both 'period' and 'start'/'end'' arguments")
@@ -160,27 +164,27 @@ class Ticker:
 
         if start is not None:
             try:
-                sched_7d = yfct.GetExchangeSchedule(exchange, start.date(), start.date()+7*td_1d)
+                sched_14d = yfct.GetExchangeSchedule(exchange, start.date(), start.date()+14*td_1d)
             except Exception as e:
                 if "Need to add mapping" in str(e):
                     raise Exception("Need to add mapping of exchange {} to xcal (ticker={})".format(self.fast_info["exchange"], self.ticker))
                 else:
                     raise
-            if sched_7d is None:
-                raise Exception("sched_7d is None for date range {}->{} and ticker {}".format(start.date(), start.date()+4*td_1d, self.ticker))
-            if sched_7d["open"][0] > dt_now:
+            if sched_14d is None:
+                raise Exception("sched_14d is None for date range {}->{} and ticker {}".format(start.date(), start.date()+14*td_1d, self.ticker))
+            if sched_14d["open"][0] > dt_now:
                 # Requested date range is in future
                 return None
         else:
-            sched_7d = None
+            sched_14d = None
 
         # All date checks passed so can begin fetching
 
         if ((start_d is None) or (end_d is None)) and (start is not None) and (end is not None):
             # if start_d/end_d not set then start/end are datetimes, so need to inspect
             # schedule opens/closes to determine days
-            if sched_7d is not None:
-                sched = sched_7d.iloc[0:1]
+            if sched_14d is not None:
+                sched = sched_14d.iloc[0:1]
             else:
                 sched = yfct.GetExchangeSchedule(exchange, start.date(), end.date()+td_1d)
             n = sched.shape[0]
@@ -201,7 +205,11 @@ class Ticker:
         else:
             h = hist.get(start, end, period=None, max_age=max_age, quiet=quiet)
         if (h is None) or h.shape[0] == 0:
-            raise Exception("history() is exiting without price data")
+            if start is not None or end is not None:
+                msg = f"YFC: history() exiting without price data (tkr={self.ticker} start={start} end={end} interval={yfcd.intervalToString[interval]})"
+            else:
+                msg = f"YFC: history() exiting without price data (tkr={self.ticker} period={period} interval={yfcd.intervalToString[interval]})"
+            raise Exception(msg)
 
         # t2_sync = perf_counter()
 
@@ -252,7 +260,7 @@ class Ticker:
                     h[c] = np.round(h[c].to_numpy(), rnd)
 
             if debug_yfc:
-                print("YFC: history() returning")
+                print("- h:")
                 cols = [c for c in ["Close", "Dividends", "Volume", "CDF", "CSF"] if c in h.columns]
                 print(h[cols])
                 if "Dividends" in h.columns:
@@ -261,9 +269,8 @@ class Ticker:
                         print("- dividends:")
                         print(h.loc[f, cols])
                 print("")
-            elif self._trace:
-                print(" "*self._trace_depth + "YFC: history() returning")
-                self._trace_depth -= 1
+            if yfcu.tc is not None:
+                yfcu.tc.Exit("Ticker::history() returning")
 
         # t4_adjust = perf_counter()
         # t_setup = t1_setup - t0
@@ -277,7 +284,7 @@ class Ticker:
         # t_cache *= 100/t_sum
         # t_filter *= 100/t_sum
         # t_adjust *= 100/t_sum
-        # print("TIME %:        setup={:.1f}%  sync={:.1f}%  filter={:.1f}%  adjust={:.1f}%".format(t_setup, t_sync, t_filter, t_adjust))
+        # print("TIME %:        setup={:.1f}%  sync={:.1f}%  filter={:.1f}%  adjust={:.1f}%".format(t_setup, t_sync, t_filter, t_adju
 
         return h
 
@@ -289,17 +296,15 @@ class Ticker:
 
         return self._histories_manager.GetHistory(interval)._getCachedPrices()
 
-
     def verify_cached_prices(self, correct=False, discard_old=False, quiet=True, debug=False, debug_interval=None):
         interval = yfcd.Interval.Days1
         cache_key = "history-"+yfcd.intervalToString[interval]
         if not yfcm.IsDatumCached(self.ticker, cache_key):
             return True
 
-        log_msg = f"YFC: _verify_cached_prices(tkr={self.ticker})"
-        if self._trace:
-            self._trace_depth += 1
-            print(" "*self._trace_depth + log_msg)
+        log_msg = f"Ticker::verify_cached_prices(tkr={self.ticker} correct={correct} quiet={quiet} debug={debug} debug_interval={debug_interval})"
+        if yfcu.tc is not None:
+            yfcu.tc.Enter(log_msg)
 
         if self._histories_manager is None:
             exchange = self.fast_info['exchange']
@@ -308,28 +313,42 @@ class Ticker:
 
         v = True
 
+        if debug_interval == yfcd.Interval.Days1:
+            debug = True
         # First verify 1d
-        # print(f"- {yfcd.intervalToString[interval]}")
         dt0 = self._histories_manager.GetHistory(interval)._getCachedPrices().index[0]
+        self.history(start=dt0.date(), quiet=quiet)  # ensure have all dividends
         v = self._verify_cached_prices_interval(interval, correct, discard_old, quiet, debug)
+        if debug_interval == yfcd.Interval.Days1:
+            if yfcu.tc is not None:
+                yfcu.tc.Exit("Ticker::verify_cached_prices() returning {v}")
+            return v
+        if not v and debug:
+            return v
         if correct:
-            # self.history(start=dt0.date())
             self.history(start=dt0.date(), quiet=quiet)
         #
         # repeat verification, because 'fetch backporting' may be buggy
         v = self._verify_cached_prices_interval(interval, correct, discard_old, quiet, debug)
+        if not v and debug:
+            return v
         if not v and correct:
             # Rows were removed so re-fetch. Only do for 1d data
-            # self.history(start=dt0.date())
             self.history(start=dt0.date(), quiet=quiet)
         #
         # verification should now pass, no exception
         v = self._verify_cached_prices_interval(interval, correct, discard_old, quiet, debug)
+        if not v and debug:
+            return v
         if not v:
             raise Exception(f"{self.ticker}: 1d failing to fetch & verify")
 
         if debug_interval is not None:
-            intervals = [debug_interval]
+            if debug_interval == yfcd.Interval.Days1:
+                intervals = []
+            else:
+                intervals = [debug_interval]
+            debug = True
         else:
             intervals = yfcd.Interval
         for interval in intervals:
@@ -342,28 +361,15 @@ class Ticker:
             # print(f"- {istr}")
             vi = self._verify_cached_prices_interval(interval, correct, discard_old, quiet, debug)
 
-            # Try a fetch and ensure re-verify. Ideally wouldn't do this, but it is finding bugs.
-            try:
-                if istr.endswith('m'):
-                    # self.history(interval=interval, period="1wk")
-                    self.history(interval=interval, period="1wk", quiet=quiet)
-                else:
-                    # self.history(interval=interval, period="1mo")
-                    self.history(interval=interval, period="1mo", quiet=quiet)
-                vi = self._verify_cached_prices_interval(interval, correct, discard_old, quiet, debug)
-            except yfcd.NoPriceDataInRangeException as e:
-                pass
-            
             v = v and vi
 
-        if self._trace:
-            print(" "*self._trace_depth + f"YFC: _verify_cached_prices() returning {v}")
-            self._trace_depth -= 1
+        if yfcu.tc is not None:
+            yfcu.tc.Exit("Ticker::verify_cached_prices() returning {v}")
 
         return v
 
     def _verify_cached_prices_interval(self, interval, correct=False, discard_old=False, quiet=True, debug=False):
-        # TODO: iterate over all intervals, but only once I'm sure verify is 100% solid. 
+        # TODO: iterate over all intervals, but only once I'm sure verify is 100% solid.
         #       - I remember needing 1d verified first => re-fetch after correction before verifying other intervals?
         if isinstance(interval, str):
             if interval not in yfcd.intervalStrToEnum.keys():
@@ -375,10 +381,9 @@ class Ticker:
         if not yfcm.IsDatumCached(self.ticker, cache_key):
             return True
 
-        log_msg = f"YFC: _verify_cached_prices_interval(tkr={self.ticker}, interval={istr})"
-        if self._trace:
-            self._trace_depth += 1
-            print(" "*self._trace_depth + log_msg)
+        log_msg = f"Ticker::_verify_cached_prices_interval(tkr={self.ticker}, interval={istr}, quiet={quiet}, debug={debug})"
+        if yfcu.tc is not None:
+            yfcu.tc.Enter(log_msg)
 
         if self._histories_manager is None:
             exchange = self.fast_info['exchange']
@@ -387,10 +392,8 @@ class Ticker:
 
         v = self._histories_manager.GetHistory(interval)._verifyCachedPrices(correct, discard_old, quiet, debug)
 
-        if self._trace:
-            print(" "*self._trace_depth + f"YFC: _verify_cached_prices_interval() returning {v}")
-            self._trace_depth -= 1
-
+        if yfcu.tc is not None:
+            yfcu.tc.Exit(f"Ticker::_verify_cached_prices_interval() returning {v}")
         return v
 
     def _process_user_dt(self, dt):
@@ -423,8 +426,6 @@ class Ticker:
         self._info = self.dat.info
         yfcm.StoreCacheDatum(self.ticker, "info", self._info)
 
-        yfct.SetExchangeTzName(self._info["exchange"], self._info["exchangeTimezoneName"])
-
         return self._info
 
     @property
@@ -441,7 +442,16 @@ class Ticker:
                 return self._fast_info
 
         # self._fast_info = self.dat.fast_info
-        self._fast_info = {k:self.dat.fast_info[k] for k in self.dat.fast_info.keys()}
+        self._fast_info = {}
+        for k in self.dat.fast_info.keys():
+            try:
+                self._fast_info[k] = self.dat.fast_info[k]
+            except Exception as e:
+                if "decrypt" in str(e):
+                    pass
+                else:
+                    print(f"TICKER = {self.ticker}")
+                    raise
         yfcm.StoreCacheDatum(self.ticker, "fast_info", self._fast_info)
 
         yfct.SetExchangeTzName(self._fast_info["exchange"], self._fast_info["timezone"])
@@ -686,7 +696,7 @@ class Ticker:
         return self._yf_lag
 
 
-def verify_cached_tickers_prices(session=None, resume_from_tkr=None, debug_tkr=None, debug_interval=None):
+def verify_cached_tickers_prices(session=None, correct=True, resume_from_tkr=None, debug_tkr=None, debug_interval=None):
     """
     :Parameters:
         session:
@@ -706,11 +716,12 @@ def verify_cached_tickers_prices(session=None, resume_from_tkr=None, debug_tkr=N
         debug_interval = yfcd.intervalStrToEnum[debug_interval]
 
     d = yfcm.GetCacheDirpath()
-    tkrs = [x for x in os.listdir(d) if not x.startswith("exchange-") and not '_' in x]
+    tkrs = [x for x in os.listdir(d) if not x.startswith("exchange-") and '_' not in x]
     # tkrs = tkrs[:5]
     # tkrs = tkrs[:20]
     # tkrs = tkrs[tkrs.index("DDOG"):]
 
+    debug = debug_tkr is not None
     if debug_tkr is not None:
         tkrs = [debug_tkr]
     else:
@@ -733,36 +744,32 @@ def verify_cached_tickers_prices(session=None, resume_from_tkr=None, debug_tkr=N
         tkr = tkrs[i]
         if tqdm_loaded:
             t.set_description("Verifying " + tkr)
+        else:
+            print(f"{tkr} : {i+1}/{len(tkrs)}")
 
-        # dat = Ticker(tkr)
         dat = Ticker(tkr, session=session)
 
-        # if debug_tkr is not None:
-        #     v = dat._verify_cached_prices(correct=True, discard_old=True, quiet=False, debug=True, debug_interval=debug_interval)
-        #     quit()
-
-        # print(">>> FIRST PASS")
         try:
-            v = dat.verify_cached_prices(correct=True, discard_old=True, quiet=True, debug_interval=debug_interval)
+            v = dat.verify_cached_prices(correct=correct, discard_old=correct, quiet=True, debug=debug, debug_interval=debug_interval)
         except yfcd.NoPriceDataInRangeException as e:
-            print(str(e) + f" - is it delisted? Aborting verification so you can investigate.")
+            print(str(e) + " - is it delisted? Aborting verification so you can investigate.")
             return
         except Exception as e:
-            print("FIRST PASS FAILED: " + str(e))
-            print("re-running with debug=True")
-            v = dat.verify_cached_prices(correct=True, discard_old=True, quiet=False, debug=True, debug_interval=debug_interval)
+            if not debug:
+                v = dat.verify_cached_prices(correct=correct, discard_old=correct, quiet=False, debug=True, debug_interval=debug_interval)
             quit()
-        # sleep(0.5)
-        # sleep(1)
-        # Second pass is important, because some cached data may have been div-adjusted without 
-        # record of that dividend. Well the first verify will re-apply that dividend, so 
+        if debug:
+            return
+        # Second pass is important, because some cached data may have been div-adjusted without
+        # record of that dividend. Well the first verify will re-apply that dividend, so
         # potential for new mismatches to correct.
-        v = dat.verify_cached_prices(correct=True, discard_old=False, quiet=True, debug_interval=debug_interval)
+        v = dat.verify_cached_prices(correct=correct, discard_old=False, quiet=True, debug=debug, debug_interval=debug_interval)
         # sleep(0.5)
         # sleep(1)
         # v = dat.verify_cached_prices(correct=True, discard_old=False, quiet=False, debug=True, debug_interval=debug_interval)
         if not v:
-            v = dat.verify_cached_prices(correct=False, discard_old=False, quiet=False, debug=True, debug_interval=debug_interval)
+            if correct:
+                v = dat.verify_cached_prices(correct=False, discard_old=False, quiet=False, debug=True, debug_interval=debug_interval)
+            else:
+                v = dat.verify_cached_prices(correct=False, discard_old=False, quiet=False, debug=False, debug_interval=debug_interval)
             raise Exception(f"{tkr}: verify failing")
-
-
